function F = normalize(F)
% NORMALIZE Locally L1 normalize the factor table
%
% F = NORMALIZE(F) is a NOOP for WORDFACTOR class.
    
% Jerod Weinman
% jerod@acm.org
% (c) 2007   

%  $Id: normalize.m 1 2009-06-17 20:04:43Z weinman $
 
% Jerod Weinman
% jerod@acm.org
% Copyright 2006, 2008, 2009, 2011
 
%    This file is part of VIDI.
%
%    VIDI is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    VIDI is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with VIDI.  If not, see <http://www.gnu.org/licenses/>.

  0;

% NO OP -- warn?
% warn('normalize operator on wordfactor is a null operation');
