function A = chararguments(F)
% CHARARGUMENTS Character node numeric names
%
%  A = CHARARGUMENTS(F)

% Jerod Weinman
% jerod@acm.org
    
% $Id: chararguments.m 1 2009-06-17 20:04:43Z weinman $
 
% Jerod Weinman
% jerod@acm.org
% Copyright 2006, 2008, 2009, 2011
 
%    This file is part of VIDI.
%
%    VIDI is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    VIDI is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with VIDI.  If not, see <http://www.gnu.org/licenses/>.

% NOTE: Relies on the fact that the factor super-class does not
% re-order arguments and the word node is the last argument to
% the factor. Otherwise, we would have to do:
% A = mysetdiff( arguments(F.factor),F.wordnode );
  
  A = arguments(F.factor);
  A = A(1:end-1);
  
